@@ -116,6 +116,145 @@ class RunPod:
     def __init__(self, gql_transport: AIOHTTPTransport):
         self.gql_transport = gql_transport
 
+    async def get_on_demand_price(self, gpuTypeId: GPUTypeId, logger: loguru.Logger, gpuCount: int = 1) -> Optional[float]:
+        """
+        Gets the current on-demand price for the gpuTypeId
+
+        :param gpuTypeId: the gpu type id to check
+        :param logger: the logger to use
+        :param gpuCount: the number of gpus to check (defaults to 1)
+        :return: the current on-demand price or None if no price is available
+        """
+        try:
+            async with Client(
+                transport=self.gql_transport,
+                fetch_schema_from_transport=False,
+            ) as session:
+                params = {
+                    "id": gpuTypeId.value,
+                    "gpuCount": gpuCount,
+                }
+
+                query = gql(
+                    """query GpuTypes ($id: String!, $gpuCount: Int!) {
+                            gpuTypes(input: {id: $id}) {
+                                secureCloud
+                                lowestPrice(input: {gpuCount: $gpuCount}) {
+                                    uninterruptablePrice
+                                }
+                            }
+                    }"""
+                )
+
+                data = await session.execute(query, variable_values=params)
+                type_data: Dict[str, str] = data["gpuTypes"][0]
+
+                if type_data["secureCloud"] and type_data["lowestPrice"]["uninterruptablePrice"] is not None:
+                    return float(type_data["lowestPrice"]["uninterruptablePrice"])
+
+                return None
+
+        except (TransportServerError, TransportQueryError) as e:
+            logger.exception(e)
+            return None
+
+    async def get_current_bid(self, gpuTypeId: GPUTypeId, logger: loguru.Logger, gpuCount: int = 1) -> Optional[float]:
+        """
+        Gets the current spot price for the gpuTypeId
+
+        :param gpuTypeId: thhe gpu type id to check
+        :param logger: the logger to use
+        :param gpuCount: the number of gpus to check (defaults to 1)
+        :return: the current bid or None if no bid is available
+        """
+        try:
+            async with Client(
+                transport=self.gql_transport,
+                fetch_schema_from_transport=False,
+            ) as session:
+                params = {
+                    "id": gpuTypeId.value,
+                    "gpuCount": gpuCount,
+                }
+
+                query = gql(
+                    """query GpuTypes ($id: String!, $gpuCount: Int!) {
+                            gpuTypes(input: {id: $id}) {
+                                communityCloud
+                                lowestPrice(input: {gpuCount: $gpuCount}) {
+                                    minimumBidPrice
+                                }
+                            }
+                    }"""
+                )
+
+                data = await session.execute(query, variable_values=params)
+                type_data: Dict[str, str] = data["gpuTypes"][0]
+
+                if type_data["communityCloud"] and type_data["lowestPrice"]["minimumBidPrice"] is not None:
+                    return float(type_data["lowestPrice"]["minimumBidPrice"])
+
+                return None
+
+        except (TransportServerError, TransportQueryError) as e:
+            logger.exception(e)
+            return None
+
+    async def get_gpu_types(self, logger: loguru.Logger) -> List[Tuple[GPUTypeId, Optional[int], bool, bool, Optional[float], Optional[float]]]:
+        """
+        Gets the gpu types and their attributes
+
+        :param logger: the logger to use
+        :return: a list of tuples containing the gpu_type_id, memoryInGb, secureCloud, communityCloud, minimum bid price, on-demand price
+        """
+        try:
+            async with Client(
+                transport=self.gql_transport,
+                fetch_schema_from_transport=False,
+            ) as session:
+                query = gql(
+                    """query GpuTypes {
+                        gpuTypes {
+                            id
+                            memoryInGb
+                            secureCloud
+                            communityCloud
+                            lowestPrice(input: {gpuCount: 1}) {
+                                minimumBidPrice
+                                uninterruptablePrice
+                            }
+                        }
+                    }"""
+                )
+
+                data = await session.execute(query)
+                types_data: List[Dict[str, Any]] = data["gpuTypes"]
+
+        except (TransportServerError, TransportQueryError) as e:
+            logger.exception(e)
+            return []
+
+        gpu_types: List[Tuple[GPUTypeId, Optional[int], bool, bool, Optional[float], Optional[float]]] = []
+        for gpuInfo in types_data:
+            try:
+                gpu_types.append(
+                    (
+                        GPUTypeId(gpuInfo["id"]),
+                        gpuInfo["memoryInGb"],
+                        gpuInfo["secureCloud"],
+                        gpuInfo["communityCloud"],
+                        gpuInfo["lowestPrice"]["minimumBidPrice"],
+                        gpuInfo["lowestPrice"]["uninterruptablePrice"],
+                    )
+                )
+            except KeyError:
+                logger.exception(f"Error parsing GPU type: {gpuInfo}")
+                continue
+
+
+
+        return gpu_types
+
     async def __create_spot_instance(
         self,
         max_bid: float,
@@ -938,6 +1077,8 @@ class RunPod:
             ):
                 logger.info(f"Pod {podId} is already running")
                 return True
+            elif "outbid" in e.errors[0]["message"]:
+                raise OutbidException(e.errors[0]["message"])
             else:
                 logger.exception(e)
                 logger.exception(query)
@@ -992,6 +1133,8 @@ class RunPod:
             ):
                 logger.info(f"Pod {podId} is already running")
                 return True
+            elif "outbid" in e.errors[0]["message"]:
+                raise OutbidException(e.errors[0]["message"])
             else:
                 logger.exception(e)
                 logger.exception(query)
